import glob
import os
import shutil

from explaining_framework.utils.io import read_yaml, write_yaml
from torch_geometric.data.makedirs import makedirs
from torch_geometric.graphgym.loader import create_dataset
from torch_geometric.graphgym.utils.io import string_to_python

if "__main__" == __name__:
    config_folder = os.path.abspath(
        os.path.join(os.path.dirname(__name__), "../../", "configs")
    )
    makedirs(config_folder)
    explaining_folder = os.path.join(config_folder, "explaining")
    makedirs(explaining_folder)
    explainer_folder = os.path.join(config_folder, "explaining")
    makedirs(explainer_folder)

    DATASET = [
        "CIFAR10",
        # "TRIANGLES",
        # "COLORS-3",
        # "REDDIT-BINARY",
        # "REDDIT-MULTI-5K",
        # "REDDIT-MULTI-12K",
        # "COLLAB",
        # "DBLP_v1",
        # "COIL-DEL",
        # "COIL-RAG",
        # "Fingerprint",
        # "Letter-high",
        # "Letter-low",
        # "Letter-med",
        "MSRC_9",
        # "MSRC_21",
        "MSRC_21C",
        # "DD",
        # "ENZYMES",
        "PROTEINS",
        # "QM9",
        # "MUTAG",
        # "Mutagenicity",
        # "AIDS",
        # "PATTERN",
        # "CLUSTER",
        "MNIST",
        "CIFAR10",
        # "TSP",
        # "CSL",
        # "KarateClub",
        # "CS",
        # "Physics",
        # "BBBP",
        # "Tox21",
        # "HIV",
        # "PCBA",
        # "MUV",
        # "BACE",
        # "SIDER",
        # "ClinTox",
        # "AIFB",
        # "AM",
        # "MUTAG",
        # "BGS",
        # "FAUST",
        # "DynamicFAUST",
        # "ShapeNet",
        # "ModelNet10",
        # "ModelNet40",
        # "PascalVOC-SP",
        # "COCO-SP",
    ]
    EXPLAINER = [
        "CAM",
        "GradCAM",
        "GNN_LRP",
        "GradExplainer",
        "GuidedBackPropagation",
        "IntegratedGradients",
        # "PGExplainer",
        "PGMExplainer",
        "RandomExplainer",
        # "SubgraphX",
        # "GraphMASK",
        "GNNExplainer",
        "EIXGNN",
        "SCGNN",
    ]

    for dataset_name in DATASET:
        for model_kind in ["best", "worst"]:
            for explainer_name in EXPLAINER:
                explaining_cfg = {}
                # explaining_cfg['adjust']['strategy']= 'rpns'
                # explaining_cfg['attack']['name']= 'all'
                explaining_cfg["cfg_dest"] = string_to_python(
                    f"dataset={dataset_name}-model={model_kind}-explainer={explainer_name}.yaml"
                )
                # = f"dataset={dataset_name}-model={model_kind}=explainer={explainer_name}-chunk=[{chunk[0]},{chunk[-1]}]"

                explaining_cfg["dataset"] = {}
                explaining_cfg["dataset"]["name"] = string_to_python(dataset_name)
                explaining_cfg["dataset"]["item"] = [3, 45, 78, 23]
                # explaining_cfg['explainer']['cfg']= 'default'
                explaining_cfg["explainer"] = {}
                explaining_cfg["explainer"]["name"] = string_to_python(explainer_name)
                explaining_cfg["explainer"]["force"] = True
                explaining_cfg["explanation_type"] = string_to_python("phenomenon")
                # explaining_cfg['metrics']['accuracy']['name']='all'
                # explaining_cfg['metrics']['fidelity']['name']='all'
                # explaining_cfg['metrics']['sparsity']['name']='all'
                explaining_cfg["model"] = {}
                explaining_cfg["model"]["ckpt"] = string_to_python(model_kind)
                explaining_cfg["model"]["path"] = string_to_python(
                    # "/media/data/SIC/araison/exps/pyg_fork/graphgym/results/graph_classif_base_grid_graph_classif_grid"
                    "/home/SIC/araison/exps/pytorch_geometric/graphgym/results/"
                    # "/media/data/SIC/araison/exps/pyg_fork/graphgym/results/graph_classif_base_grid_graph_classif_grid"
                )
                # explaining_cfg['out_dir']='./explanation'
                # explaining_cfg['print']='both'
                # explaining_cfg['threshold']['config']['type']='all'
                # explaining_cfg['threshold']['value']['hard']=[0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
                # explaining_cfg['threshold']['value']['topk']=[2, 3, 5, 10, 20, 30, 50]
                PATH = os.path.join(
                    explaining_folder + "/" + explaining_cfg["cfg_dest"],
                )
                write_yaml(explaining_cfg, PATH)
    # if os.path.exists(PATH):
    # continue
    # else:
    # write_yaml(explaining_cfg, PATH)
    # configs = [
    # path for path in glob.glob(os.path.join(explaining_folder, "**", "*.yaml"))
    # ]
    # for path in configs:
    # data = read_yaml(path)
    # data["model"][
    # "path"
    # ] = "/media/data/SIC/araison/exps/pyg_fork/graphgym/results/graph_classif_base_grid_graph_classif_grid"
    # write_yaml(data, path)

    # for index, config_chunk in enumerate(
    # chunkizing_list(configs, int(len(configs) / 5))
    # ):
    # PATH_ = os.path.join(explaining_folder, f"gpu={index}")
    # makedirs(PATH_)
    # for path in config_chunk:
    # filename = os.path.basename(path)
    # shutil.copy2(path, os.path.join(PATH_, filename))
