import copy

import torch
from torch import FloatTensor
from torch.nn import ReLU
from torch_geometric.explain.explanation import Explanation


class Adjust(object):
    def __init__(
        self,
        strategy: str = "rpns",
    ):
        self.strategy = strategy

    def forward(self, exp: Explanation) -> Explanation:
        exp_ = exp.clone()
        for k, v in exp_.items():
            if "mask" in k:
                for f_ in self.strategy:
                    if f_ == "r":
                        exp_.__setattr__(k, self.relu(v))
                    if f_ == "a":
                        exp_.__setattr__(k, self.absolute(v))
                    if f_ == "p":
                        if "edge" in k:
                            pass
                        else:
                            exp_.__setattr__(k, self.project(v))
                    if f_ == "n":
                        exp_.__setattr__(k, self.normalize(v))
                    if f_ == "s":
                        exp_.__setattr__(k, self.squeeze_(v))

            else:
                continue

        return exp_

    def relu(self, mask: FloatTensor) -> FloatTensor:
        relu = ReLU(inplace=True)
        mask_ = relu(mask)
        return mask_

    def squeeze_(self, mask: FloatTensor) -> FloatTensor:
        if mask.max() == mask.min():
            return mask
        else:
            mask_ = (mask - mask.min()).div(mask.max() - mask.min())
        return mask_

    def normalize(self, mask: FloatTensor) -> FloatTensor:
        norm = torch.norm(mask, p=float("inf"))
        if norm.item() > 0:
            mask_ = mask / norm.item()
            return mask_
        else:
            return mask

    def project(self, mask: FloatTensor) -> FloatTensor:
        if mask.ndim >= 2:
            mask_ = torch.sum(mask, dim=1)
            return mask_
        else:
            return mask

    def absolute(self, mask: FloatTensor) -> FloatTensor:
        mask_ = torch.abs(mask)
        return mask_
